@@ -94,15 +94,39 @@ class KhtTextEdit(QtGui.QPlainTextEdit):
         fr = self.frameRect()
         cr = self.contentsRect()
         self.setMinimumHeight(max(s.height(), s.height() + (fr.height() - cr.height() - 1)))
-        self.setMinimumWidth(s.width())
+        print s.width(),s.width() + (fr.width()-cr.width()) - 1
+        self.setMinimumWidth(max(s.width(),s.width() + (fr.width()-cr.width()) - 1))
+
+    def ensureVisible(self,pos,xmargin,ymargin):
+
+        visible = self.area.viewport().size()        
+        currentPos = QtCore.QPoint(self.area.horizontalScrollBar().value(),
+                      self.area.verticalScrollBar().value())
+        posRect = QtCore.QRect(pos.x()-xmargin, pos.y()-ymargin,2*xmargin,2*ymargin)
+        visibleRect = QtCore.QRect(currentPos, visible)
+
+        if (visibleRect.contains(posRect)):
+            return
+
+        newPos = currentPos
+        if (posRect.top() < visibleRect.top()):
+            newPos.setY(posRect.top())
+        elif (posRect.bottom() > visibleRect.bottom()):
+            newPos.setY(posRect.bottom() - visible.height())
+        if (posRect.left() < visibleRect.left()):
+            newPos.setX(posRect.left())
+        elif (posRect.right() > visibleRect.right()):
+            newPos.setX(posRect.right() - visible.width())
+        self.scroller.scrollTo(newPos)
+
 
     def curPositionChanged(self):
         #Hilight current line
         self.highlightCurrentLine()
         
         #Make sure cursor is visible
-        cursor = self.cursorRect()
-        pos = self.pos()
+#        cursor = 
+#        pos = self.pos()
         #scroller = self.parentWidget().property("kineticScroller").toPyObject() 
         #pos = self.parent().mapToParent(pos) 
         #print type(self.parent()), type(self.parentWidget()),type(scroller)
@@ -110,7 +134,10 @@ class KhtTextEdit(QtGui.QPlainTextEdit):
         #self.scroller.ensureVisible(pos + cursor.center(), 10 + cursor.width(),
         #                       2 * cursor.height())
 #        pw = self
-        self.scroller.ensureVisible(cursor.center(), 2*cursor.width(), 2*cursor.height())
+        cursor = self.cursorRect()
+        pos = cursor.center()
+#        print cursor.center(),cursor.center().x(),cursor.center().y()
+        self.area.ensureVisible(pos.x(),pos.y(), 2*cursor.width()+20, 2*cursor.height())
 #        while (pw):
 #            if (pw.parentWidget()):
 #                area = pw.parentWidget()
